# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 Andre Hugo cortexcoder@gmail.com
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import gettext
from gettext import gettext as _
gettext.textdomain('framingham')

import gtk
import logging
logger = logging.getLogger('framingham')

from framingham_lib import Window
from framingham.AboutFraminghamDialog import AboutFraminghamDialog
from framingham.PreferencesFraminghamDialog import PreferencesFraminghamDialog
from framingham.ResultDialog import ResultDialog

# See framingham_lib.Window.py for more details about how this class works
class FraminghamWindow(Window):
    __gtype_name__ = "FraminghamWindow"
    
    def finish_initializing(self, builder): # pylint: disable=E1002
        """Set up the main window"""
        super(FraminghamWindow, self).finish_initializing(builder)

        self.AboutDialog = AboutFraminghamDialog
        self.PreferencesDialog = PreferencesFraminghamDialog

        # Code for other initialization actions should be added here.
        disclaimer = "Disclaimer\n\nWarning: Please note that this application is not a substitute for good clinical practice. "
        disclaimer += "\nUse of this application is at own risk.\n\n"
        disclaimer += "Framingham Calculator \n"
        disclaimer += "is based on Wilson, D'Agostino, Levy et al.\n"
        disclaimer += "'Prediction of Coronary Heart Disease using Risk Factor Categories', Circulation 1998"

        self.builder.get_object("disclaimer").set_text(disclaimer)


    #change diclaimer background area to white
    def expose(self, widget, event):
        cr = widget.window.cairo_create()
        cr.set_source_rgb(255, 255, 255)
        cr.rectangle(event.area.x, event.area.y,
                     event.area.width, event.area.height)
        cr.fill()

    def metrics_changed(self, widget, data=None):

        # Change mmol/l to mg/dl and vice versa
        if (self.builder.get_object("radiobuttonmg").get_active()):
            self.builder.get_object("combobox3").set_model(self.builder.get_object("cholestrolmg"))
            self.builder.get_object("combobox3").set_active(0)
            self.builder.get_object("combobox4").set_model(self.builder.get_object("LDLmg"))
            self.builder.get_object("combobox4").set_active(0)
            self.builder.get_object("combobox5").set_model(self.builder.get_object("HDLmg"))
            self.builder.get_object("combobox5").set_active(0)
        else:
            self.builder.get_object("combobox3").set_model(self.builder.get_object("cholestrol"))
            self.builder.get_object("combobox3").set_active(0)
            self.builder.get_object("combobox4").set_model(self.builder.get_object("LDL"))
            self.builder.get_object("combobox4").set_active(0)
            self.builder.get_object("combobox5").set_model(self.builder.get_object("HDL"))
            self.builder.get_object("combobox5").set_active(0)
        

    def on_buttoncalculate_clicked(self, widget, data=None):

        #Set male scoring lists
        age_male_list = (-1,0,1,2,3,4,5,6,7)
        LDL_male_list = (0,-3,0,0,1,2) #0 is placeholder for no data
        Cholesterol_male_list = (0,-3,0,1,2,3) #0 is placeholder for no data
        HDL_male_list_LDL = (
2,1,0,0,-1)
        HDL_male_list_chol = (2,1,0,0,-2)
        Hypertension_male_list = (0,0,1,2,3)
        Diabetic_male_list = (0,2)
        Smoking_male_list = (0,2)

        #Set female scoring lists
        age_female_list = (-9,-4,0,3,6,7,8,8,8)
        LDL_female_list = (0,-2,0,0,2,2) #0 is placeholder for no data
        Cholesterol_female_list = (0,-2,0,1,1,3) #0 is placeholder for no data
        HDL_female_list_LDL = (5,2,1,0,-2)
        HDL_female_list_chol = (5,2,1,0,-3)
        Hypertension_female_list = (-3,0,0,2,3)
        Diabetic_female_list = (0,4)
        Smoking_female_list = (0,2)

        #Set Risk persentages
        LDL_men = (1,2,2,3,4,4,6,7,9,11,14,18,22,27,33,40,47,56)
	    Chol_men = (0,0,2,3,3,4,5,7,8,10,13,16,20,25,31,37,45,53)
        LDL_female = (1,2,2,2,3,3,4,5,6,7,8,9,11,13,15,17,17,20,24,27,32)
	    Chol_female = (1,2,2,2,3,3,4,4,5,6,7,8,10,11,13,15,15,18,20,24,27)

        #Set average risk persentages
        Men_average = (3,5,7,11,14,16,21,25,30)
	    Men_average_hard = (1,4,4,8,10,13,20,22,25)
	    Men_average_low = (2,3,4,4,6,7,9,11,14)
        Women_average = ("<1","<1","2","5","8","12","12","13","14")
	    Women_average_hard = ("<1","<1","1","2","3","7","8","8","11")
	    Women_average_low = ("<1","1","2","3","5","7","8","8","8")

        # Retrieve ComboBox and radio buttons state
        age = self.builder.get_object("combobox1").get_active()
        bp = self.builder.get_object("combobox2").get_active()
        chol = self.builder.get_object("combobox3").get_active()
        LDL = self.builder.get_object("combobox4").get_active()
        HDL = self.builder.get_object("combobox5").get_active()
        smoking = self.builder.get_object("radiobuttonsmokingyes").get_active()
        diabetic = self.builder.get_object("radiobuttondiabeticyes").get_active()

        #Set intial result text
        resulttext = "\nFramingham Risk using:"

        score = 0
        score_LDL = 0
        score_chol = 0 

        if (self.builder.get_object("radiobuttonmale").get_active()): #male calculation

            

            #Get Smoking and Diabetic values
            if (self.builder.get_object("radiobuttonsmokingyes").get_active()):
                smoking_score = Smoking_male_list[1]
            else:
                smoking_score = Smoking_male_list[0]
            print "Smoking score: " + str(smoking_score)

            if (self.builder.get_object("radiobuttondiabeticyes").get_active()):
                diabetic_score = Diabetic_male_list[1]
            else:
                diabetic_score = Diabetic_male_list[0]
            print "Diabetic score: " + str(diabetic_score)

            #Start calculation based on the LDL or Cholesterol pathways

            #Cholesterol and LDL Pathway
            if (chol != 0 and LDL != 0):
                score_LDL += age_male_list[age] + LDL_male_list[LDL] + HDL_male_list_LDL[HDL] + Hypertension_male_list[bp] + smoking_score + diabetic_score
                score_chol += age_male_list[age] + Cholesterol_male_list[chol] + HDL_male_list_chol[HDL] + Hypertension_male_list[bp] + smoking_score + diabetic_score
                if (score_LDL >= 14):
                    risk_LDL = LDL_men[17]
                elif (score_LDL < -3):
                    risk_LDL = LDL_men[0]
                else:
                    risk_LDL = LDL_men[score_LDL + 3]
                if (score_chol >= 14):
                    risk_chol = Chol_men[17]
                elif (score_chol < -3):
                    risk_chol = Chol_men[0]
                else:
                    risk_chol = Chol_men[score_chol + 3]

                resulttext += "\t\tCholesterol: " + str(risk_chol) + "%"
                resulttext += "\t\tLDL: " +str(risk_LDL) + "%"

            #LDL Pathway
            elif (LDL !=0):
                score_LDL += age_male_list[age] + LDL_male_list[LDL] + HDL_male_list_LDL[HDL] + Hypertension_male_list[bp] + smoking_score + diabetic_score
                if (score_LDL >= 14):
                    risk_LDL = LDL_men[17]
                elif (score_LDL < -3):
                    risk_LDL = LDL_men[0]
                else:
                    risk_LDL = LDL_men[score_LDL + 3]

                resulttext += "\t\tLDL: " +str(risk_LDL) + "%"

            #Cholesterol Pathway
            elif (chol != 0):
                score_chol += age_male_list[age] + Cholesterol_male_list[chol] + HDL_male_list_chol[HDL] + Hypertension_male_list[bp] + smoking_score + diabetic_score
                if (score_chol >= 14):
                    risk_chol = Chol_men[17]
                elif (score_chol < -3):
                    risk_chol = Chol_men[0]
                else:
                    risk_chol = Chol_men[score_chol + 3]

                resulttext += "\t\tCholesterol: " + str(risk_chol) + "%"

            #Nothing selected error handeling
            elif (chol == 0 and LDL == 0):
                resulttext = "\n\n\nYou have not selected a Cholesterol or LDL value.\nPlease select an value in one or both selection boxes.\n\n\n"


            #Get comparative risk scores of related age
            compare_risk_low = Men_average_low[age]
            compare_risk_average = Men_average[age]
            compare_risk_hard = Men_average_hard[age]

            if (chol !=0 or LDL !=0):
                resulttext += "\n\nCompared Risk:\t\tLow Risk\t " + str(compare_risk_low) + "%\t\tAverage Risk\t " + str(compare_risk_average) + "%\t\tHard Risk\t " + str(compare_risk_hard) + "%"    
                resulttext += "\n\nHard Cardiovascular events excludes Angina Pectoris"
                resulttext += "\n\nLow Risk was calculated with the following:"
                resulttext += "\n\n\t\t\t\t\t1. Same age\n\t\t\t\t\t2. Optimal Blood Pressure\n\t\t\t\t\t3. LDL of 100-129mg/dl\n\t\t\t\t\t4. Cholesterol 160-199mg/dl"
                resulttext += "\n\t\t\t\t\t5. HDL 45 mg/dl (men) 55mg/dl (female)\n\t\t\t\t\t6. Non Smoker\n\t\t\t\t\t7. Non Diabetic"

            print "Age: " + str(age_male_list[age]) + "\nLDL: " + str(LDL_male_list[LDL]) + "\nHDL (LDL): " + str(HDL_male_list_LDL[HDL]) + "\nHDL (chol): " + str(HDL_male_list_chol[HDL]) + "\nBP: " + str(Hypertension_male_list[bp])
            print "Chol: " +str(Cholesterol_male_list[chol])
            print "Score: " + str(score) + " Chol: " + str(score_chol) + " LDL: " + str(score_LDL)

        else: #female calculation


            #Get Smoking and Diabetic values
            if (self.builder.get_object("radiobuttonsmokingyes").get_active()):
                smoking_score = Smoking_female_list[1]
            else:
                smoking_score = Smoking_female_list[0]
            print "Smoking score: " + str(smoking_score)

            if (self.builder.get_object("radiobuttondiabeticyes").get_active()):
                diabetic_score = Diabetic_female_list[1]
            else:
                diabetic_score = Diabetic_female_list[0]
            print "Diabetic score: " + str(diabetic_score)

            #Start calculation based on the LDL or Cholesterol pathways

            #Cholesterol and LDL Pathway
            if (chol != 0 and LDL != 0):
                score_LDL += age_female_list[age] + LDL_female_list[LDL] + HDL_female_list_LDL[HDL] + Hypertension_female_list[bp] + smoking_score + diabetic_score
                score_chol += age_female_list[age] + Cholesterol_female_list[chol] + HDL_female_list_chol[HDL] + Hypertension_female_list[bp] + smoking_score + diabetic_score
                print "both: " + str(score_chol)
                if (score_LDL >=17):
                    risk_LDL = LDL_female[20]
                elif (score_LDL <= -2):
                    risk_LDL = LDL_female[0]
                else:
                    risk_LDL = LDL_female[score_LDL + 2]
                if (score_chol >=17):
                    risk_chol = Chol_female[20]
                elif (score_chol <= -2):
                    risk_chol = Chol_female[0]
                else:
                    risk_chol = Chol_female[score_chol + 2]
                resulttext += "\t\tCholesterol: " + str(risk_chol) + "%"
                resulttext += "\t\tLDL: " +str(risk_LDL) + "%"

            #LDL pathway
            elif (LDL !=0):
                score_LDL += age_female_list[age] + LDL_female_list[LDL] + HDL_female_list_LDL[HDL] + Hypertension_female_list[bp] + smoking_score + diabetic_score
                if (score_LDL >=17):
                    risk_LDL = LDL_female[20]
                elif (score_LDL <= -2):
                    risk_LDL = LDL_female[0]
                else:
                    risk_LDL = LDL_female[score_LDL + 2]

                resulttext += "\t\tLDL: " +str(risk_LDL) + "%"

            #Cholesterol Pathway
            elif (chol != 0):
                score_chol += age_female_list[age] + Cholesterol_female_list[chol] + HDL_female_list_chol[HDL] + Hypertension_female_list[bp] + smoking_score + diabetic_score
                if (score_chol >=17):
                    risk_chol = Chol_female[20]
                elif (score_chol <= -2):
                    risk_chol = Chol_female[0]
                else:
                    risk_chol = Chol_female[score_chol + 2]

                resulttext += "\t\tCholesterol: " + str(risk_chol) + "%"

            #Nothing selected error handeling
            elif (chol == 0 and LDL == 0):
                resulttext = "\n\n\nYou have not selected an Cholesterol or LDL value.\nPlease select an value in one or both selection boxes.\n\n\n"

            #Get comparative risk scores of related age
            compare_risk_low = Women_average_low[age]
            compare_risk_average = Women_average[age]
            compare_risk_hard = Women_average_hard[age]

            if (chol !=0 or LDL !=0):
                resulttext += "\n\nCompared Risk:\t\tLow Risk\t " + str(compare_risk_low) + "%\t\tAverage Risk\t " + str(compare_risk_average) + "%\t\tHard Risk\t " + str(compare_risk_hard) + "%"
                resulttext += "\n\nHard Cardiovascular events excludes Angina Pectoris"
                resulttext += "\n\nLow Risk was calculated with the following:"
                resulttext += "\n\n\t\t\t\t\t1. Same age\n\t\t\t\t\t2. Optimal Blood Pressure\n\t\t\t\t\t3. LDL of 100-129mg/dl\n\t\t\t\t\t4. Cholesterol 160-199mg/dl"
                resulttext += "\n\t\t\t\t\t5. HDL 45 mg/dl (men) 55mg/dl (female)\n\t\t\t\t\t6. Non Smoker\n\t\t\t\t\t7. Non Diabetic"

            print "Age: " + str(age_female_list[age]) + "\nLDL: " + str(LDL_female_list[LDL]) + "\nHDL (LDL): " + str(HDL_female_list_LDL[HDL]) + "\nHDL (chol): " + str(HDL_female_list_chol[HDL]) + "\nBP: " + str(Hypertension_female_list[bp])
            print "Chol: " +str(Cholesterol_female_list[chol])
            print "Score: " + str(score) + " Chol: " + str(score_chol) + " LDL: " + str(score_LDL)



        dialog = ResultDialog()
        dialog.builder.get_object("dialogtext").set_text(resulttext)
        response = dialog.run()
        dialog.destroy()
        if response == -5:
            return False # returning False makes "destroy-event" be signalled
                         # for the window.
        else:
            return True # returning True avoids it to signal "destroy-event"

